<?php

namespace Drupal\commerce_ecommpay;

use GuzzleHttp\ClientInterface;
use ecommpay\SignatureHandler;
use ecommpay\Payment;
use ecommpay\Gate;
use ecommpay\exception\ValidationException;
use Drupal\address\AddressInterface;

/**
 * Client for ECOMMpay API.
 */
class EcommpayClient {

  const API_HOST = 'https://api.ecommpay.com';

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  private $client;

  /**
   * Constructor.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The HTTP client.
   */
  public function __construct(ClientInterface $client) {
    $this->client = $client;
  }

  /**
   * Returns payment page URL.
   *
   * @param string $project_id
   *   Project ID.
   * @param string $secret_key
   *   Secret key.
   * @param string $payment_id
   *   Payment ID.
   * @param string $customer_id
   *   Customer ID.
   * @param string $email
   *   Customer email.
   * @param \Drupal\address\AddressInterface $billing_address
   *   Billing address.
   * @param int $amount
   *   Amount in cents.
   * @param string $currency_code
   *   Currency code.
   * @param string $success_url
   *   Success URL.
   * @param string $failure_url
   *   Failure URL.
   * @param string $notify_url
   *   Notify URL.
   * @param string $language_code
   *   Language.
   * @param string $card_operation_type
   *   Card operation type.
   *
   * @return string
   *   Payment page URL.
   */
  public function getPaymentPageUrl(
    $project_id,
    $secret_key,
    $payment_id,
    $customer_id,
    $email,
    ?AddressInterface $billing_address,
    $amount,
    $currency_code,
    $success_url,
    $failure_url,
    $notify_url,
    $language_code,
    $card_operation_type = 'sale') {
    // Looks like we need to pass.
    $ecommpay_payment = new Payment($project_id, $payment_id);
    $ecommpay_payment->setPaymentAmount($amount)
      ->setPaymentCurrency($currency_code)
      ->setLanguageCode($language_code)

      // Set operation type.
      ->setCardOperationType($card_operation_type)

      // Sort out redirect urls and modes.
      ->setMerchantSuccessUrl($success_url)
      ->setMerchantFailUrl($failure_url)
      ->setRedirectSuccessUrl($success_url)
      ->setRedirectFailUrl($failure_url)
      ->setRedirectSuccessMode('parent_page')
      ->setRedirectFailMode('parent_page')

      // Set callback url.
      ->setMerchantCallbackUrl($notify_url)

      // Customer details.
      ->setCustomerId($customer_id)
      ->setCustomerEmail($email);

    if (!empty($billing_address)) {
      $ecommpay_payment
        ->setCardHolder(implode(
          ' ',
          array_filter([$billing_address->getGivenName(),
            $billing_address->getFamilyName(),
          ])
        ))
        ->setCustomerFirstName($billing_address->getGivenName())
        ->setCustomerLastName($billing_address->getFamilyName())
        ->setCustomerAddress($billing_address->getAddressLine1())
        ->setCustomerCity($billing_address->getLocality())
        ->setCustomerCountry($billing_address->getCountryCode())
        ->setCustomerPostal($billing_address->getPostalCode())
        ->setCustomerRegion($billing_address->getAdministrativeArea())

        ->setBillingFirstName($billing_address->getGivenName())
        ->setBillingLastName($billing_address->getFamilyName())
        ->setBillingAddress($billing_address->getAddressLine1())
        ->setBillingCity($billing_address->getLocality())
        ->setBillingCountry($billing_address->getCountryCode())
        ->setBillingPostal($billing_address->getPostalCode())
        ->setBillingRegion($billing_address->getAdministrativeArea());
    }

    $gate = new Gate($secret_key);

    try {
      $url = $gate->getPurchasePaymentPageUrl($ecommpay_payment);
    }
    catch (ValidationException $e) {
      \Drupal::logger('ecommpay')->error($e->getFormattedMessage());
      throw $e;
    }

    return $url;
  }

  /**
   * Refunds payment.
   *
   * @param string $project_id
   *   Project ID.
   * @param string $secret_key
   *   Secret key.
   * @param string $payment_id
   *   Payment ID.
   * @param int $amount
   *   Amount in cents.
   * @param string $currency_code
   *   Currency code.
   *
   * @return mixed
   *   Response.
   */
  public function refundPayment($project_id, $secret_key, $payment_id, $amount, $currency_code) {
    $signature_handler = new SignatureHandler($secret_key);

    // API data.
    $url = self::API_HOST . '/v2/payment/card/refund';
    $data = [
      "general" => [
        "project_id" => $project_id,
        "payment_id" => $payment_id,
      ],
      "payment" => [
        "description" => "refund",
        "amount" => $amount,
        "currency" => $currency_code,
      ],
    ];
    $data['general']['signature'] = $signature_handler->sign($data);

    // Post data to URL.
    $response = $this->client->post($url, [
      'json' => $data,
    ]);

    $response = json_decode($response->getBody()->getContents(), TRUE);

    // Check if status is present.
    if (empty($response['status'])) {
      throw new \Exception('Invalid response');
    }

    return $response;
  }

  /**
   * Captures payments.
   *
   * @param string $project_id
   *   Project ID.
   * @param string $secret_key
   *   Secret key.
   * @param string $payment_id
   *   Payment ID.
   * @param int $amount
   *   Amount in cents.
   * @param string $currency_code
   *   Currency code.
   *
   * @return mixed
   *   Response.
   */
  public function capturePayment($project_id, $secret_key, $payment_id, $amount, $currency_code) {
    $signature_handler = new SignatureHandler($secret_key);

    // API data.
    $url = self::API_HOST . '/v2/payment/card/capture';
    $data = [
      "general" => [
        "project_id" => $project_id,
        "payment_id" => $payment_id,
      ],
      "payment" => [
        "description" => "capture",
        "amount" => $amount,
        "currency" => $currency_code,
      ],
    ];
    $data['general']['signature'] = $signature_handler->sign($data);

    // Post data to URL.
    $response = $this->client->post($url, [
      'json' => $data,
    ]);

    $response = json_decode($response->getBody()->getContents(), TRUE);

    // Check if status is present.
    if (empty($response['status'])) {
      throw new \Exception('Invalid response');
    }

    return $response;
  }

  /**
   * Voids payments.
   *
   * @param string $project_id
   *   Project ID.
   * @param string $secret_key
   *   Secret key.
   * @param string $payment_id
   *   Payment ID.
   *
   * @return mixed
   *   Response.
   */
  public function voidPayment($project_id, $secret_key, $payment_id) {
    $signature_handler = new SignatureHandler($secret_key);

    // API data.
    $url = self::API_HOST . '/v2/payment/card/cancel';
    $data = [
      "general" => [
        "project_id" => $project_id,
        "payment_id" => $payment_id,
      ],
    ];
    $data['general']['signature'] = $signature_handler->sign($data);

    // Post data to URL.
    $response = $this->client->post($url, [
      'json' => $data,
    ]);

    $response = json_decode($response->getBody()->getContents(), TRUE);

    // Check if status is present.
    if (empty($response['status'])) {
      throw new \Exception('Invalid response');
    }

    return $response;
  }

  /**
   * Returns payment state.
   *
   * @param string $project_id
   *   Project ID.
   * @param string $secret_key
   *   Secret key.
   * @param string $payment_id
   *   Payment ID.
   *
   * @return string
   *   Payment state.
   */
  public function getPaymentState($project_id, $secret_key, $payment_id) {
    $signature_handler = new SignatureHandler($secret_key);

    // Form data.
    $url = self::API_HOST . '/v2/payment/status';
    $data = [
      "general" => [
        "project_id" => $project_id,
        "payment_id" => $payment_id,
      ],
    ];
    $data['general']['signature'] = $signature_handler->sign($data);

    // Post data to URL.
    $response = $this->client->post($url, [
      'json' => $data,
    ]);

    $response = json_decode($response->getBody()->getContents(), TRUE);

    // Check if payment status is present.
    if (empty($response['payment']['status'])) {
      throw new \Exception('Invalid response');
    }

    return $response['payment']['status'];
  }

  /**
   * Check validity of signature.
   *
   * @param string $secret_key
   *   Secret key.
   * @param mixed $data
   *   Data to validate.
   * @param string $signature
   *   Signature.
   *
   * @return bool
   *   TRUE if signature is valid, FALSE otherwise.
   */
  public function checkSignature($secret_key, $data, $signature) {
    $signature_handler = new SignatureHandler($secret_key);
    return $signature_handler->check($data, $signature);
  }

  /**
   * Parses callback body.
   *
   * @param string $secret_key
   *   Secret key.
   * @param string $request_body
   *   Request body.
   *
   * @return \ecommpay\Callback
   *   Callback object.
   */
  public function parseCallbackBody($secret_key, $request_body) {
    // Create callback object with raw data.
    $gate = new Gate($secret_key);
    $callback = $gate->handleCallback($request_body);

    // Check integrity by testing the signature.
    if (!$callback->checkSignature()) {
      \Drupal::logger('commerce_ecommpay')->error('Invalid signature when processing callback');
      throw new \Exception('Invalid signature.');
    }

    return $callback;
  }

}
